﻿Imports Microsoft.VisualBasic
Imports System
Imports Microsoft.SolverFoundation.Services ' C:\Program Files (x86)\Reference Assemblies\Microsoft\Framework\.NETFramework\v4.0

Namespace SudokuSolver

    Friend Class SudokuProgram

        Shared Sub Main(ByVal args() As String)

            Console.WriteLine(vbLf & "Begin MS Solver Foundation Sudoku demo" & vbLf)
            Console.WriteLine("The problem is: " & vbLf)
            Dim data(8)() As Integer

            data(0) = New Integer() {0, 0, 6, 2, 0, 0, 0, 8, 0}
            data(1) = New Integer() {0, 0, 8, 9, 7, 0, 0, 0, 0}
            data(2) = New Integer() {0, 0, 4, 8, 1, 0, 5, 0, 0}

            data(3) = New Integer() {0, 0, 0, 0, 6, 0, 0, 0, 2}
            data(4) = New Integer() {0, 7, 0, 0, 0, 0, 0, 3, 0}
            data(5) = New Integer() {6, 0, 0, 0, 5, 0, 0, 0, 0}

            data(6) = New Integer() {0, 0, 2, 0, 4, 7, 1, 0, 0}
            data(7) = New Integer() {0, 0, 3, 0, 2, 8, 4, 0, 0}
            data(8) = New Integer() {0, 5, 0, 0, 0, 1, 2, 0, 0}

            For r As Integer = 0 To 8

                For c As Integer = 0 To 8

                    If data(r)(c) = 0 Then
                        Console.Write(" _")
                    Else
                        Console.Write(" " & data(r)(c))
                    End If

                Next c
                Console.WriteLine("")

            Next r

            Dim problem As SolverContext = SolverContext.GetContext()
            Dim model As Model = problem.CreateModel()

            Dim grid(8)() As Decision ' 0-based indices, 1-based values
            For r As Integer = 0 To 8
                grid(r) = New Decision(8) {}
            Next r

            For r As Integer = 0 To 8
                For c As Integer = 0 To 8
                    grid(r)(c) = New Decision(Domain.IntegerRange(1, 9), "grid" & r & c)
                Next c
            Next r

            For r As Integer = 0 To 8
                model.AddDecisions(grid(r))
            Next r

            'For r As Integer = 0 To 8 ' alternative to above
            '    For c As Integer = 0 To 8
            '        model.AddDecisions(grid(r)(c))
            '    Next c
            'Next r

            Console.WriteLine(vbLf & "Creating generic row constraints")
            For r As Integer = 0 To 8
                model.AddConstraint("rowConstraint" & r, model.AllDifferent(grid(r)))
            Next r

            Console.WriteLine("Creating generic column constraints")
            For c As Integer = 0 To 8

                For first As Integer = 0 To 7
                    For second As Integer = first + 1 To 8
                        model.AddConstraint("colConstraint" & c & first & second, grid(first)(c) <> grid(second)(c))
                    Next second
                Next first

            Next c

            Console.WriteLine("Creating generic sub-cube constraints")
            ' cube constraints for grid[a][b] and grid[x][y]
            For r As Integer = 0 To 8 Step 3

                For c As Integer = 0 To 8 Step 3

                    For a As Integer = r To r + 3 - 1

                        For b As Integer = c To c + 3 - 1

                            For x As Integer = r To r + 3 - 1

                                For y As Integer = c To c + 3 - 1

                                    If (x = a AndAlso y > b) OrElse (x > a) Then ' xy > ab

                                        model.AddConstraint("cubeConstraint" & a & b & x & y, grid(a)(b) <> grid(x)(y))

                                    End If

                                Next y ' y

                            Next x ' x

                        Next b ' b

                    Next a ' a

                Next c ' c

            Next r ' r

            Console.WriteLine("Creating problem specific data constraints")

            AddDataConstraints(data, model, grid) ' more elegant approach

            Console.WriteLine(vbLf & "Solving. . . ")

            Dim numSolutions As Integer = NumberSolutions(problem)
            Console.WriteLine(vbLf & "There is/are " & numSolutions & " Solution(s)" & vbLf)

            Dim solution As Solution = problem.Solve()
            'ShowAnswer(grid); // alternative to below

            For r As Integer = 0 To 8

                For c As Integer = 0 To 8

                    Dim v As Double = grid(r)(c).GetDouble()
                    Console.Write(" " & v)

                Next c
                Console.WriteLine("")

            Next r

            Console.WriteLine(vbLf & "End Solver Foundation Sudoku demo" & vbLf)
            Console.ReadLine()

        End Sub ' Main

        Private Shared Sub AddDataConstraints(ByVal data()() As Integer, ByVal model As Model, ByVal grid()() As Decision)

            For r As Integer = 0 To 8

                For c As Integer = 0 To 8

                    If data(r)(c) <> 0 Then

                        model.AddConstraint("v" & r & c, grid(r)(c) = data(r)(c))

                    End If

                Next c

            Next r

        End Sub

        Private Shared Function NumberSolutions(ByVal problem As SolverContext) As Integer

            Dim ct As Integer = 0
            Dim soln As Solution = problem.Solve()
            Do While soln.Quality = SolverQuality.Feasible

                ct += 1
                soln.GetNext()

            Loop
            Return ct

        End Function

    End Class ' Program

End Namespace ' ns